import yaml
from test import assertEquals

try:    
    for test in yaml.loadFile("./TestingSuite/ypath.yml"):
        if not test.has_key('ignore'):
            expr = test['ypath']
            pth = yaml.ypath(expr,cntx=1)
            lst = [] 
            for x in pth.apply(test['data']):
                lst.append(str(x))
            exp = test['expected']
            if test.has_key('unordered'):
               lst.sort()
               exp.sort()
            assertEquals(lst,exp,expr + " => " + str(pth))

    print "Experimental YPATH OK"
except NotImplementedError: 
    print "Experimental YPATH requires Python 2.2"



import types
import string
from types import StringType, UnicodeType, IntType, FloatType
from types import DictType, ListType, TupleType, InstanceType
from yaml.klass import hasMethod, isDictionary
import re

"""
  The methods from this module that are exported to the top 
  level yaml package should remain stable.  If you call
  directly into other methods of this module, be aware that 
  they may change or go away in future implementations.
  Contact the authors if there are methods in this file 
  that you wish to remain stable.
"""

def dump(*data):
    return Dumper().dump(*data)

def d(data): return dump(data)

def dumpToFile(file, *data):
    return Dumper().dumpToFile(file, *data)

class Dumper:
    def __init__(self):
        self.currIndent   = "\n"
        self.indent = "    "
        self.keysrt   = None
        self.alphaSort = 1 # legacy -- on by default

    def setIndent(self, indent):
        self.indent = indent
        return self

    def setSort(self, sort_hint):
        self.keysrt = sortMethod(sort_hint)
        return self

    def dump(self, *data):
        self.result = []  
        self.output = self.outputToString
        self.dumpDocuments(data)
        return string.join(self.result,"")

    def outputToString(self, data):
        self.result.append(data)

    def dumpToFile(self, file, *data):
        self.file = file
        self.output = self.outputToFile
        self.dumpDocuments(data)

    def outputToFile(self, data):
        self.file.write(data)

    def dumpDocuments(self, data):
        for obj in data:
            self.anchors  = YamlAnchors(obj)
            self.output("---")
            self.dumpData(obj)
            self.output("\n")       

    def indentDump(self, data):
        oldIndent = self.currIndent
        self.currIndent += self.indent
        self.dumpData(data)
        self.currIndent = oldIndent

    def dumpData(self, data):
        anchor = self.anchors.shouldAnchor(data)
        if anchor: 
            self.output(" &%d" % anchor )
        else:
            anchor = self.anchors.isAlias(data)
            if anchor:
                self.output(" *%d" % anchor )
                return
        if (data is None):
            self.output(' ~')
        elif hasMethod(data, 'to_yaml'):
            self.dumpTransformedObject(data)            
        elif hasMethod(data, 'to_yaml_implicit'):
            self.output(" " + data.to_yaml_implicit())
        elif type(data) is InstanceType:
            self.dumpRawObject(data)
        elif isDictionary(data):
            self.dumpDict(data)
        elif type(data) in [ListType, TupleType]:
            self.dumpList(data)
        else:
            self.dumpScalar(data)

    def dumpTransformedObject(self, data):
        obj_yaml = data.to_yaml()
        if type(obj_yaml) is not TupleType:
            self.raiseToYamlSyntaxError()
        (data, typestring) = obj_yaml
        if typestring:
            self.output(" " + typestring)
        self.dumpData(data)

    def dumpRawObject(self, data):
        self.output(' !!%s.%s' % (data.__module__, data.__class__.__name__))
        self.dumpData(data.__dict__)

    def dumpDict(self, data):
        keys = data.keys()
        if len(keys) == 0:
            self.output(" {}")
            return
        if self.keysrt:
            keys = sort_keys(keys,self.keysrt)
        else:
            if self.alphaSort:
                keys.sort()
        for key in keys:
            self.output(self.currIndent)
            self.dumpKey(key)
            self.output(":")
            self.indentDump(data[key])

    def dumpKey(self, key):
        if type(key) is TupleType:
            self.output("?")
            self.indentDump(key) 
            self.output("\n")
        else:
            self.output(quote(key))

    def dumpList(self, data):
        if len(data) == 0:
            self.output(" []")
            return
        for item in data:
            self.output(self.currIndent)
            self.output("-")
            self.indentDump(item)

    def dumpScalar(self, data):
        if isUnicode(data):
            self.output(' "%s"' % repr(data)[2:-1])
        elif isMulti(data):
            self.dumpMultiLineScalar(data.splitlines())
        else:
            self.output(" ")
            self.output(quote(data))
    
    def dumpMultiLineScalar(self, lines):
        self.output(" |")
        if lines[-1] == "":
            self.output("+")
        for line in lines:
            self.output(self.currIndent)
            self.output(line)

    def raiseToYamlSyntaxError(self):
            raise """
to_yaml should return tuple w/object to dump 
and optional YAML type.  Example:
({'foo': 'bar'}, '!!foobar')
"""

#### ANCHOR-RELATED METHODS

def accumulate(obj,occur):
    typ = type(obj)
    if obj is None or \
       typ is IntType or \
       typ is FloatType or \
       ((typ is StringType or typ is UnicodeType) \
       and len(obj) < 32): return
    obid = id(obj)
    if 0 == occur.get(obid,0):
        occur[obid] = 1
        if typ is ListType:
            for x in obj: 
                accumulate(x,occur)
        if typ is DictType:
            for (x,y) in obj.items():
                accumulate(x,occur)
                accumulate(y,occur)
    else:
        occur[obid] = occur[obid] + 1

class YamlAnchors:
     def __init__(self,data):
         occur = {}
         accumulate(data,occur)
         anchorVisits = {}
         for (obid, occur) in occur.items():
             if occur > 1:
                 anchorVisits[obid] = 0 
         self._anchorVisits = anchorVisits
         self._currentAliasIndex     = 0
     def shouldAnchor(self,obj):
         ret = self._anchorVisits.get(id(obj),None)
         if 0 == ret:
             self._currentAliasIndex = self._currentAliasIndex + 1
             ret = self._currentAliasIndex
             self._anchorVisits[id(obj)] = ret
             return ret
         return 0
     def isAlias(self,obj):
         return self._anchorVisits.get(id(obj),0)

### SORTING METHODS

def sort_keys(keys,fn):
    tmp = []
    for key in keys:
        val = fn(key)
        if val is None: val = '~'
        tmp.append((val,key))
    tmp.sort()
    return [ y for (x,y) in tmp ]

def sortMethod(sort_hint):
    typ = type(sort_hint)
    if DictType == typ:
        return sort_hint.get
    elif ListType == typ or TupleType == typ:
        indexes = {}; idx = 0
        for item in sort_hint:
            indexes[item] = idx
            idx += 1
        return indexes.get
    else:
        return sort_hint

### STRING QUOTING AND SCALAR HANDLING
def isStr(data):
    # XXX 2.1 madness
    if type(data) == type(''):
        return 1
    if type(data) == type(u''):
        return 1
    return 0
    
def doubleUpQuotes(data):
    return data.replace("'", "''")

def quote(data):
    if not isStr(data):
        return str(data)
    single = "'"
    double = '"'
    quote = ''
    if len(data) == 0:
        return "''"
    if hasSpecialChar(data) or data[0] == single:
        data = `data`[1:-1]
        data = string.replace(data, r"\x08", r"\b")
        quote = double 
    elif needsSingleQuote(data):
        quote = single
        data = doubleUpQuotes(data)
    return "%s%s%s" % (quote, data, quote)

def needsSingleQuote(data):
    if re.match(r"^-?\d", data):
        return 1
    if re.match(r"\*\S", data):
        return 1
    if data[0] in ['&', ' ']:
        return 1
    if data[0] == '"':
        return 1
    if data[-1] == ' ':
        return 1
    return (re.search(r'[:]', data) or re.search(r'(\d\.){2}', data))

def hasSpecialChar(data):
    # need test to drive out '#' from this
    return re.search(r'[\t\b\r\f#]', data)

def isMulti(data):
    if not isStr(data):
        return 0
    if hasSpecialChar(data):
        return 0
    return re.search("\n", data)

def isUnicode(data):
    return type(data) == unicode
    
def sloppyIsUnicode(data):
        # XXX - hack to make tests pass for 2.1
        return repr(data)[:2] == "u'" and repr(data) != data

import sys
if sys.hexversion < 0x20200000:
    isUnicode = sloppyIsUnicode
    





import YamlTest
from here import *
from test import assertEquals
from types import DictType, ListType, TupleType, InstanceType

"""
This is experimental code.  It generates simulated events
for a pull-based YAML parser.
"""

class MockEmitter:
    """
    A pushed-based emitter may be more awkward to write, 
    because you have to keep track of more state.
    """
    def __init__(self):
        self.events = []

    def append(self, *event):
        self.events.append(event)        

    def pushScalar(self, data):
        self.append('getType', 'scalar')
        self.append('getScalar', data)

    def appendType(self, typ):
        self.append('getType', typ)

    def startSeq(self):
        self.appendType('seq')
        
    def endSeq(self):
        self.appendType(None)

    def startMap(self):
        self.appendType('map')
        
    def endMap(self):
        self.appendType(None)

class Dumper:
    def __init__(self, emitter):
        self.emitter = emitter

    def dump(self, data):
        if type(data) in (ListType, TupleType):
            return self.dumpList(data)
        elif type(data) is DictType:
            return self.dumpDict(data)
        else:
            self.emitter.pushScalar(data)

    def dumpList(self, data):
        self.emitter.startSeq()
        for item in data:
            self.dump(item)
        self.emitter.endSeq()

    def dumpDict(self, data):
        self.emitter.startMap()
        keys = data.keys()
        keys.sort() # XXX - only for now
        for key in keys:
            value = data[key]
            self.dump(key)
            self.dump(value)
        self.emitter.endMap()

def mockEvents(data):
    emitter = MockEmitter()
    dumper = Dumper(emitter)
    dumper.dump(data)
    return emitter.events

class Test(YamlTest.YamlTest):

    def testScalar(self):
        assertEquals(mockEvents('foobar'), self.scalar('foobar'))

    def scalar(self, data):
        return [ 
            ('getType', 'scalar'),
            ('getScalar', data)
        ]

    def list123(self):
        (result) = (
            [('getType', 'seq' )] +
            self.scalar(1) +
            self.scalar(2) +
            self.scalar(3) +
            [( 'getType', None )])
        return result

    def testList(self):
        assertEquals(mockEvents([1,2,3]), self.list123())

    def dictFoobar(self):
        (result) = (
            [('getType', 'map')] +
            self.scalar('bar') +
            self.scalar(2) + 
            self.scalar('foo') +
            self.scalar(1) +
            [( 'getType', None)])
        return result

    def testDict(self):
        events = mockEvents({'foo': 1, 'bar': 2})
        assertEquals(events, self.dictFoobar())

    def testListDict(self):
        events = mockEvents([ [1,2,3], {'foo': 1, 'bar': 2} ])
        self.assertEquals(
            events,
            [('getType', 'seq' )] +
            self.list123() +
            self.dictFoobar() + 
            [('getType', None)])            

if __name__ == '__main__':
    import unittest
    unittest.main()



import yaml
from test import assertEquals
import re

def Query(query, doc):
    results = []
    for row in yaml.ypath(query['from'],doc):
        data = {}
        if Where(row, query):
            for field in query['select']:
                data[field] = yaml.ypath(field,row).next()
            if query.has_key('order'):
                data['__row__'] = row
            results.append(data)
    return Order(results, query)

def Where(row, query):
    if query.has_key('where'):
        where = query['where']
        return WhereRow(row, query['where'])
    return 1

def WhereRow(row, where):
    if type(where) == type([]):
        return And(row, where, WhereRow)
    if type(where) == type(''):
        return WhereString(row, where)
    return And(row, where.keys(), 
        lambda row, field: where[field] == row[field])

def And(row, conds, condFunc):
    for cond in conds:
        if not condFunc(row, cond):
            return 0
    return 1

def WhereString(row, where):
    where = re.sub("\[(.*?)\]", r"row['\1']", where)
    return eval(where)

def Order(results, query):
    if not query.has_key('order'):
        return results
    compare = lambda x,y: Compare(x,y,query['order'])
    results.sort(compare)
    for result in results:
        del result['__row__']
    return results

def Compare(x, y, order):
    for field in order:
        val = cmp(
            yaml.ypath(field,x['__row__']).next(),
            yaml.ypath(field,y['__row__']).next()
        )
        if val != 0: return val

def TestQuery(query, expected, data):
    assertEquals(Query(query, data), expected, query)    

doc = yaml.loadFile("query.yml").next()
for test in doc['tests']:
    if not test.has_key('ignore'):
        TestQuery(test['query'], test['expected'], doc['data'])





import yaml

def writeFile(fn, data):
    f = open(fn, 'w')
    f.write(data)

writeFile('toplevel.yml', """
    - doc1: !!include doc1.yml
    - doc2: !!include doc2.yml
    """)

writeFile('doc1.yml', "foo: bar")
writeFile('doc2.yml', "more: stuff")

class Includer:
    def resolveType(self, value, url):
        return yaml.loadFile(value).next()

print yaml.loadFile('toplevel.yml', Includer()).next()



import YamlTest
from here import *
from test import assertEquals
from yaml.load import l
from yaml.dump import d

class Test(YamlTest.YamlTest):
    def setUp(self):
        # print '>>>>>>>>>>>>>>>'
        pass

    def testNoLineFeed(self):
        from yaml.stream import noLineFeed
        assertEquals("foo", noLineFeed("foo\n"))
        assertEquals("bar", noLineFeed("bar\n"))
        assertEquals("bar", noLineFeed("bar\r\n"))
        assertEquals("", noLineFeed("\r\n"))

    def XXXtest9(self):
        # XXX - not sure how this gets handled now
        self.verify(
            """
            - foo: 1
                bar: 2
            """,
            [ {'foo': 1, '  bar': 2}])

    def testOutlineSnippet(self):
        self.verify(
            """
            - YAML ToDo:
                - y2outline
                - support generic transfers
                - work on YAML.py:
                    - work on Store
            """,
            [ {'YAML ToDo': [   
                'y2outline',
                'support generic transfers',
                { 'work on YAML.py': ['work on Store'] }
                ]
              }
            ])

    def testQuotedString(self):
        self.verify(
            """
            ---
            foo: 'key: value'
            """,
            {'foo': "key: value"}
            )

    def testFloatRedHerring(self):
        self.verify(
            """
            ---
            - 'Version: 0.18.0'
            """,
            [ 'Version: 0.18.0' ]
            )

    def testDashInQuotes(self):
        self.verify(
            """
            ---
            title: 'Perspective Broker - twisted.spread'
            """,
            {'title': 'Perspective Broker - twisted.spread'}
            )

    def testQuestionMarks(self):
        self.verify(
            """
            ---
            hello: you there?
            foo:
                - ok?
                - sure?
            bar:
                - for here or to go?
                - more sugar?
            """,
            {   
                'hello': 'you there?',
                'foo': ['ok?', 'sure?'],
                'bar': ['for here or to go?', 'more sugar?']
            }
            )

    def testDoubleQuoteEscapedKeys(self):
        self.verify(
            """
            ---
            "I'm escaped!": simple
            """,
            { "I'm escaped!": 'simple' }
            )

    def testColonsInsideEscapedKeys(self):
        self.verify(
            """
            ---
            'aaa: bbbb': simple
            """,
            { 'aaa: bbbb': 'simple' }
            )

    def testControlChars(self):
        self.verify(
            r"""
            control: "\b1998\t1999\t2000\n"
            """,
            { 'control': "\b1998\t1999\t2000\n" }
        )

    def testUnicode(self):
        if YamlTest.hasUnicode:
            self.verify(
                r'''
                unicode: "Sosa did fine.\u263A"
                ''',
                { 'unicode': u"Sosa did fine.\u263A"}
            )

    def testMultiLineScalar(self):
        self.verify(
            """
            plain: This unquoted
                   scalar spans
                   many lines.
            """,
            { 'plain': 'This unquoted scalar spans many lines.' }
        )

    def testDomainType(self):
        class MyYamlConfig:
            def resolveType(self, data, url):
                if url == '!!name':
                    return 'Foo ' + data
                elif url == '!!coords':
                    return { 'x': data[0], 'y': data[1]}
                else:
                    raise 'url not passed in correctly'
        data = YamlTest.loadFlushLeft("""
            name: !!name Barson
            coords: !!coords
              - 10
              - 20
            """, 
            MyYamlConfig())
        self.assertEquals(data[0], 
            {'name': 'Foo Barson',
            'coords': { 'x': 10, 'y': 20 } }
        )

    def testQuickInterfaces(self):
        assertEquals(l("--- foo"), "foo")
        assertEquals(d({'foo': 'bar'}), "---\nfoo: bar\n")

if __name__ == '__main__':
    import unittest
    unittest.main()



import YamlTest
from test import assertEquals
from yaml.load import *


class Test(YamlTest.YamlTest):
    def testSimple(self):
        tokenizer = InlineTokenizer('[ foo, bar ]')
        assertEquals(tokenizer.next(), '[')
        assertEquals(tokenizer.next(), 'foo')
        assertEquals(tokenizer.next(), 'bar')
        assertEquals(tokenizer.next(), ']')

    def testSimpleHash(self):
        tokenizer = InlineTokenizer('{ foo: bar }')
        assertEquals(tokenizer.next(), '{')
        assertEquals(tokenizer.next(), 'foo: bar') # might change
        assertEquals(tokenizer.next(), '}')

if __name__ == '__main__':
    import unittest
    unittest.main()



import YamlTest
from yaml.load import *
from here import flushLeft
from test import assertEquals
from yaml.stream import NestedText, CommentEater, split

here = flushLeft # pardon the Perlism

def stringStream(str):
    return StringStream(here(str))

def commentEater(str):
    return CommentEater(stringStream(str))

def nestedText(str):
    return NestedText(stringStream(str))

def nestedDocs(str):
    return NestedDocs(stringStream(str))

class Test(YamlTest.YamlTest):
    def tryPop(self, expected):
        assertEquals(self.nt.pop(), expected)
        
    def tryPopNestedLines(self, expected):
        assertEquals(self.nt.popNestedLines(), expected)
        
    def tryPeek(self, expected):
        assertEquals(self.nt.peek(), expected)

    def test1(self):
        self.nt = nestedText(
            """
            1a
             2a
             2b
                 3a
             2c
            1b
            """)
        self.tryPop('1a')
        self.nt.nestToNextLine()
        self.tryPop('2a')
        self.tryPop('2b')
        self.nt.nestToNextLine()
        self.tryPop('3a')
        self.tryPop(None)
        self.tryPop('2c')
        self.tryPop(None)
        self.tryPeek('1b')
        self.tryPop('1b')
        self.tryPop('')
        self.tryPop(None)
        
    def test2(self):
        self.nt = nestedText("""
            apple
                banana
                   foo
            """)
        self.tryPop('apple')
        self.nt.nestToNextLine()
        self.tryPop('banana')
        self.nt.nestBySpecificAmount(2)
        self.tryPop(' foo')

    def test3(self):
        self.nt = nestedDocs("""
            seattle
            --- foo
            bluffton
            """)
        assertEquals(self.nt.popDocSep(), '---')
        self.tryPop('seattle')
        self.tryPop(None)
        assertEquals(self.nt.popDocSep(), '--- foo')
        self.tryPop('bluffton')
        self.tryPop(None)

    def testEatComments(self):
        self.nt = commentEater("""
            # ignore
              # these
            silly
            """)
        self.tryPop('silly')

    def testNestedDocsEatComments(self):
        self.nt = nestedDocs("""
            # foo
              # bar
            ---
            shabazz
            """)
        self.nt.popDocSep()
        self.tryPop('shabazz')
        assertEquals(self.nt.lastLineRead(), 4)

    def testNestedTextEatNewline(self):
        self.nt = nestedDocs("""
            
            ---
            shabazz
            """)
        self.nt.popDocSep()
        self.tryPop('shabazz')

    def testDocSep(self):
        data = here("""
            --- city
            seattle
            --- town
            bluffton
            """)
        self.nt = NestedDocs(StringStream(data))
        assertEquals(self.nt.popDocSep(), '--- city')
        self.tryPop('seattle')
        self.tryPop(None)
        assertEquals(self.nt.popDocSep(), '--- town')
        self.tryPop('bluffton')
        self.tryPop(None)

        data = here("""
            # comments at top
            # should be ignored
            """) + data
        self.nt = NestedDocs(StringStream(data))
        assertEquals(self.nt.popDocSep(), '--- city')
        self.tryPop('seattle')

    def testNestedComments(self):
        self.nt = nestedText("""
            apple
              # ignore this comment
                banana
                   foo
            """)
        self.tryPop('apple')
        self.nt.nestToNextLine()
        self.tryPop('banana')
        self.nt.nestBySpecificAmount(2)
        self.tryPop(' foo')

    def testStream(self):
        stream = StringStream(here("""
            python
            perl
            java
            """))
        assertEquals(stream.get(), "python")
        assertEquals(stream.get(), "perl")
        assertEquals(stream.get(), "java")
        
    def test1(self):
        self.nt = nestedText(
            """
            1a
             2a
             2b
                 3a
             2c
            1b
            """)
        self.tryPop('1a')
        self.nt.nestToNextLine()
        self.tryPop('2a')
        self.tryPop('2b')
        self.nt.nestToNextLine()
        self.tryPop('3a')
        self.tryPop(None)
        self.tryPop('2c')
        self.tryPop(None)
        self.tryPeek('1b')
        self.tryPop('1b')
        self.tryPop(None)
        
    def XXXtestEmptyLines(self):
        # not sure if this is valid test any more
        self.nt = nestedDocs("""
            ---
             apple

                banana
            """)
        self.nt.popDocSep()
        self.nt.nestToNextLine()
        self.tryPop('apple')
        self.nt.nestToNextLine()
        self.tryPop('')
        self.nt.nestToNextLine()
        self.tryPop('banana')

    def testFoldedCase(self):
        self.nt = nestedText(
            """
            ---
             Aaa
             Bbb

               11
               22

             Ccc
            """)
        self.nt.pop()
        self.nt.nestToNextLine()
        self.tryPop('Aaa')
        self.tryPop('Bbb')
        self.tryPop('')

    def testRedHerringDocSep(self):
        self.nt = nestedDocs(
            """
            ---
            foo:
                ---
            """)
        self.nt.popDocSep()
        self.tryPop('foo:')
        self.nt.nestToNextLine()
        self.tryPop('---')

    def testGetToNextIndent(self):
        self.nt = nestedText(
            """
            ---
            line1: this is
               continued
            line2
            """)
        self.nt.pop()
        self.nt.nestToNextLine()
        self.tryPop('line1: this is')
        self.tryPopNestedLines(['continued'])
        self.tryPop('line2')
            
    def testGetToNextIndent(self):
        self.nt = nestedText(
            """
            ---
            key1: this is
               continued
               with multiple lines
            key2:
                 key2a:
                   hello
                   world
            key3:
            """)
        self.nt.pop()
        self.nt.nestToNextLine()
        self.tryPop('key1: this is')
        self.tryPopNestedLines(['continued', 'with multiple lines'])
        self.tryPop('key2:')
        self.nt.nestToNextLine()
        self.tryPop('key2a:')
        self.tryPopNestedLines(['hello', 'world'])
            
    def testPopNestedLinesWithNoneNested(self):
        self.nt = nestedText(
            """
            ---
            - apple
            - banana
            - carrot
            """)
        self.nt.pop()
        self.nt.nestToNextLine()
        self.tryPop('- apple')
        self.tryPopNestedLines([])
        self.tryPop('- banana')

    def testLineNumbers(self):
        data = \
            """
            one
            two
            #
            three
            """
        stream = StringStream(here(data))
        stream.get()
        assertEquals(stream.lastLineRead(), 1)
        stream.get()
        assertEquals(stream.lastLineRead(), 2)
        nt = nestedText(data)
        nt.pop()
        assertEquals(nt.lastLineRead(), 1)
        nt.pop()
        assertEquals(nt.lastLineRead(), 2)
        nt.pop()  # eats a comment too
        assertEquals(nt.lastLineRead(), 4)

    def testSplit(self):
        assertEquals(split("foo\nbar"), ['foo', 'bar'])
        assertEquals(split("foo\nbar\n"), ['foo', 'bar'])        

    def testLoaderException(self):
        import yaml
        from yaml.stream import YamlLoaderException
        exception = None
        try:
            yaml.load('invalid YAML').next()
        except YamlLoaderException, e:
            exception = e
        assertEquals(exception.lineNum, 1)
            

if __name__ == '__main__':
    import unittest
    unittest.main()



#!/usr/bin/env python
import yaml, re
try:
    print yaml.loadFile('TestFileNamesInErrors.py').next()
    raise "YAML actually parsed this file, uh oh!"
except Exception, e:
    if not re.search('TestFileNamesInErrors\.py', str(e)):
        raise 'YAML not reporting filename properly ("%s")' % e



__version__ = "0.32"
from load import loadFile, load, Parser, l
from dump import dump, dumpToFile, Dumper, d
from stream import YamlLoaderException, StringStream, FileStream
from timestamp import timestamp
import sys
if sys.hexversion >= 0x02020000:
    from redump import loadOrdered

try:
    from ypath import ypath
except NameError:
    def ypath(expr,target='',cntx=''):
        raise NotImplementedError("ypath requires Python 2.2")

if sys.hexversion < 0x02010000:
    raise 'YAML is not tested for pre-2.1 versions of Python'



import yaml
from yaml.ordered_dict import OrderedDict
from yaml.redump import redump
from test import assertEquals

stream = """\
---
zzz: 0
yyy: 1
xxx: 2
alpha: 3
---
-
    foo: 1
    bar: 2
-
    z: 1
    y: 2
"""

my_dict = yaml.loadOrdered(stream).next()
assertEquals(my_dict.keys(), ['zzz', 'yyy', 'xxx', 'alpha'])

assertEquals(redump(stream), stream)
   



from yaml.ordered_dict import OrderedDict
from yaml import Parser, Dumper, StringStream

def loadOrdered(stream):
    parser = Parser(StringStream(stream))
    parser.dictionary = OrderedDict
    return iter(parser)

def redump(stream):
    docs = list(loadOrdered(stream))
    dumper = Dumper()
    dumper.alphaSort = 0
    return dumper.dump(*docs)
    



#!/usr/bin/env python
"""
Sample code to convert YAML to XML and XML to YAML using a canonical
form.

The canonical YAML representation of an XML element is a
dictionary (mapping) containing the following key/value pairs:
    (1) "name" (required) -- a string.
    (2) "attributes" (optional) -- a dictionary (mapping) of name/value
        pairs.
    (3) "text" (optional) -- a string.
    (4) "children" (optional) -- a sequence of dictionaries (mappings).

For usage information, type:
    python convertyaml_map.py

Usage: python convertyaml_map.py <option> <in_file>

Options:
    -y2x    Convert YAML file to XML document.
    -x2y    Convert XML document to YAML.


Requirements:
    PyXML
    PyYaml

"""

import sys, string, re, types
import yaml

from xml.dom import minidom
from xml.dom import Node


#
# Convert a YAML file to XML and write it to stdout.
#
def convertYaml2Xml(inFileName):
    inobj = yaml.loadFile(inFileName)
    out = []
    level = 0
    convertYaml2XmlAux(inobj, level, out)
    outStr = "".join(out)
    sys.stdout.write(outStr)


def convertYaml2XmlAux(inobj, level, out):
    for obj in inobj:
        if type(obj) == types.DictType and \
            obj.has_key('name'):
            name = obj['name']
            attributes = None
            if obj.has_key('attributes'):
                attributes = obj['attributes']
            text = None
            if obj.has_key('text'):
                text = obj['text']
            children = None
            if obj.has_key('children'):
                children = obj['children']
            addLevel(level, out)
            out.append('<%s' % name)
            if attributes:
                for key in attributes:
                    out.append(' %s="%s"' % (key, attributes[key]))
            if not (children or text):
                out.append('/>\n')
            else:
                if children:
                    out.append('>\n')
                else:
                    out.append('>')
                if text:
                    out.append(text)
                if children:
                    convertYaml2XmlAux(children, level + 1, out)
                    addLevel(level, out)
                out.append('</%s>\n' % name)


#
# Convert an XML document (file) to YAML and write it to stdout.
#
def convertXml2Yaml(inFileName):
    doc = minidom.parse(inFileName)
    root = doc.childNodes[0]
    # Convert the DOM tree into "YAML-able" data structures.
    out = convertXml2YamlAux(root)
    # Ask YAML to dump the data structures to a string.
    outStr = yaml.dump(out)
    # Write the string to stdout.
    sys.stdout.write(outStr)


def convertXml2YamlAux(obj):
    objDict = {}
    # Add the element name.
    objDict['name'] = obj.nodeName
    # Convert the attributes.
    attrs = obj.attributes
    if attrs.length > 0:
        attrDict = {}
        for idx in range(attrs.length):
            attr = attrs.item(idx)
            attrDict[attr.name] = attr.value
        objDict['attributes'] = attrDict
    # Convert the text.
    text = []
    for child in obj.childNodes:
        if child.nodeType == Node.TEXT_NODE and \
            not isAllWhiteSpace(child.nodeValue):
            text.append(child.nodeValue)
    if text:
        textStr = "".join(text)
        objDict['text'] = textStr
    # Convert the children.
    children = []
    for child in obj.childNodes:
        if child.nodeType == Node.ELEMENT_NODE:
            obj = convertXml2YamlAux(child)
            children.append(obj)
    if children:
        objDict['children'] = children
    return objDict


#
# Utility functions.
#
def addLevel(level, out):
    for idx in range(level):
        out.append('    ')


NonWhiteSpacePattern = re.compile('\S')

def isAllWhiteSpace(text):
    if NonWhiteSpacePattern.search(text):
        return 0
    return 1


USAGE_TEXT = """
Convert a file from YAML to XML or XML to YAML and write it to stdout.

Usage: python convertyaml_map.py <option> <in_file>

Options:
    -y2x    Convert YAML file to XML document.
    -x2y    Convert XML document to YAML.
"""

def usage():
    print USAGE_TEXT
    sys.exit(-1)


def main():
    args = sys.argv[1:]
    if len(args) != 2:
        usage()
    option = args[0]
    inFileName = args[1]
    if option == '-y2x':
        convertYaml2Xml(inFileName)
    elif option == '-x2y':
        convertXml2Yaml(inFileName)
    else:
        usage()


if __name__ == '__main__':
    main()
    #import pdb
    #pdb.run('main()')




import unittest, sys, string
import yaml
from here import flushLeft
import re
import glob
from test import assertEquals, assertError
from yaml.klass import makeClass

try:
   unicode('')
   hasUnicode = 1
except:
   hasUnicode = 0


def loadFlushLeft(stream, typeResolver=None):
    return list(yaml.load(flushLeft(stream), typeResolver))

class YamlTest(unittest.TestCase):

    def runTest(self):
        # reinvent the wheel, instead of trying to 
        # understand unittest's braindead interface
        for name in dir(self.__class__):
            if re.match('test', name):
                exec('self.%s()' % name)

    def verify(self, stream, results):
        self.verifyMany(stream, [results])

    def verifyMany(self, stream, expected):
        results = loadFlushLeft(stream)
        assertEquals(results, expected)

class ClassForRuntime:
    def __init__(self):
        self.foo = 1
        self.bar = 2

class YamlLoader:
    # XXX python 2.1 has weak lambda support
    def __init__(self, doc):
        self.doc = doc
    def __call__(self):
        return list(yaml.load(self.doc['yaml']))

class TestFromSpec(YamlTest):
    def testFromYaml(self):
        testFiles = glob.glob('TestingSuite/*.yml')
        for file in testFiles:
            # print "\n\n\n\n", file
            try:
                docs = yaml.loadFile(file)
            except IOError:
                raise "See TESTING for how to set up TestingSuite"
            for doc in docs:
                if doc.has_key('python'):
                    self.verifyOneDoc(doc)
                if doc.has_key('error') and doc['error'].has_key('python'):
                    assertError(YamlLoader(doc), doc['error']['python'])

    def verifyOneDoc(self, doc):
        if doc.has_key('python_setup'):
            exec(doc['python_setup'])
        data = eval(doc['python'])
        # print data
        assertEquals(list(yaml.load(doc['yaml'])), data)
        if not doc.has_key('NO_ROUND_TRIP'):
            self.assertRoundTrip(data)

    def assertRoundTrip(self, data):
        # XXX - A5 won't round trip when enclosed by 
        # an array...need to investigate
        for item in data:
            # print dump(item)
            assertEquals(item, 
                yaml.load(yaml.dump(item)).next(), 'roundtrip')


class TestUtils(YamlTest):    
    class Dummy:
        pass

    def testMakeClass(self):
        self.assertEquals(42,
            makeClass('YamlTest', 'TestUtils.Dummy', 
                {'a': 27, 'b': 42}).b)

    class FromYaml:
        def from_yaml(self, dict):
            self.sum = dict['a'] + dict['b']
            return self

    def testMakeClassFromYaml(self):
        self.assertEquals(6,
            makeClass('YamlTest', 'TestUtils.FromYaml',
                {'a': 2, 'b': 4}).sum)


def testVersion():
    from yaml import __version__
    assertEquals(__version__, str(yaml.loadFile('CHANGELOG').next()[0]['version']))

if __name__ == '__main__':
    import TestYamlBasics
    import TestNestedText
    import TestInlineTokenizer
    import TestDumper
    ts = unittest.TestSuite()
    ts.addTest(TestYamlBasics.Test())
    ts.addTest(TestNestedText.Test())
    ts.addTest(TestInlineTokenizer.Test())
    ts.addTest(TestDumper.Test())
    ts.addTest(TestFromSpec())
    ts.addTest(TestUtils())
    unittest.TextTestRunner().run(ts)
    yaml.dumpToFile(sys.stdout, {'TESTING dumpToFile': 'ok'})
    testVersion()



import YamlTest
from here import flushLeft
from test import assertEquals
from TestPushDumper import mockEvents
from yaml import load

"""
This is experimental code.  I am moving toward a rewrite of the YAML 
parser so that it uses a pull interface.  I am using a mock object
to emulate the parser, so that I can experiment with the interface a bit,
and also to sketch out some ideas for schema-driven parsing.
"""

class MockParser:
    def __init__(self, events):
        self.events = events
        self.index = 0

    def __getattr__(self, name):
        (nextName, value) = self.events[self.index]
        if name != nextName:
            raise "Improper mocking for event %d (%s vs. %s)" % \
                (self.index, name, nextName)
        self.index += 1
        return lambda: value

class Loader:
    def __init__(self, parser):
        self.parser = parser

    def load(self, data):
        typ = self.parser.getType()
        return self._load(typ)

    def _load(self, typ):
        if typ == 'seq':
            return self._loadSeq()
        elif typ == 'map':
            return self._loadMap()
        else:
            return self._loadScalar()

    def _loadSeq(self):
        results = []
        def itemFunc(self, results, typ):
            results.append(self._load(typ))
        return self.iterateItems(results, itemFunc)

    def _loadMap(self):
        results = {}
        def itemFunc(self, results, typ):
            key = self._load(typ)
            valTyp = self.parser.getType()
            value = self._load(typ)
            results[key] = value
        return self.iterateItems(results, itemFunc)

    def iterateItems(self, results, func):
        while 1:
            typ = self.parser.getType()
            if typ is None:
                return results
            else:
                func(self, results, typ)

    def _loadScalar(self):
        return self.parser.getScalar()

def mockParser(data):
    events = mockEvents(data)
    return MockParser(events)

def mockLoad(data):
    loader = Loader(mockParser(data))
    return loader.load(None) # None for data cause events are all mocked 

def testRoundTrip(data):
    obj = mockLoad(data)
    assertEquals(obj, data)

class Test(YamlTest.YamlTest):

    def testMock(self):
        parser = MockParser( [
            ('getScalar', 'foo'),
            ('getArray', [1,2,3]),
            ('getScalar', None),
        ])
        assertEquals(parser.getScalar(), 'foo')
        assertEquals(parser.getArray(), [1,2,3])
        assertEquals(parser.getScalar(), None)

    def testList(self):
        testRoundTrip([1,2,3])

    def testDict(self):
        testRoundTrip({'foo': 'bar'})

    def testListDict(self):
        testRoundTrip( [ [1,2,3], {'foo': 'bar'} ] )

    def testComplexStructure(self):
        data = load(flushLeft(
            """
            list:
                - {foo: bar}
                - [1, 2, 3]
            dict:
                name: steve
                games: [hoops, pool]
            """))
        testRoundTrip(data)

if __name__ == '__main__':
    import unittest
    unittest.main()



import re
from string import split, join

def flushLeft(s):
    lines = split(s,'\n')
    lastLine = lines.pop()
    if re.match(r"\S", lastLine):
        raise "bad last line"
    indent = len(lastLine)
    ret = []
    for line in lines[1:]:
        ret.append(line[indent:] + '\n')
    return join(ret,'')




import YamlTest
from here import flushLeft
from test import assertEquals, assertError
from TestPullParser import mockParser, Loader
from yaml import load

"""
Part of the parse/pull experimental code.  This does a schema-driven
validating parse of YAML documents, but it's built on top of a 
crufty interim solution.

The schema-driven parser requires a pull parser interface.  Ideally 
a pull-parser would be pulling nodes from a YAML document on an
as-needed basis, and this is the eventual goal.  But, I don't have
a pull parser yet, so I simulate one by reading in the entire YAML
document into a Python data structure, then I do a push-based dump 
of the data structure to a mock emitter that stores up a list of
parser events that a mock parser then serves up to the schema-driven
parser on an as-needed basis.  Sounds complex, but there's really not
much code involved.

Nothing fancy is supported yet--just lists, dictionaries, and scalars; 
no aliases, class transformations, multiple docs, etc.  Also, we lose
the sort order on map keys, so you will notice that all the examples
have alphabetically sorted keys.

Also, once we go to a true pull parser, we could have more metadata,
such as line numbers for nodes, attached comments, etc., that can 
help with error reporting and round-tripping issues.
"""

testCases = """
-
    data: |
        --- foo
    schema:
        type: scalar
-
    data: |
        --- foo
    schema:
        type: seq
    error: |
        Wanted seq, got scalar
 -
    data: &list123 |
        ---
        - 1
        - 2
        - 3
    schema:
        type: seq
        child:
            type: scalar
  -
    data: *list123
    schema:
        type: seq
        max: 2
        child:
            type: scalar
    error: |
        Seq has max 2 elements
-
    data: |
        ---
        city: New Orleans
        state: LA
        street: Bourbon
    schema: &StreetCityState
        type: map
        items:
            - name: city
              value:
                  type: scalar
            - name: state
              value:
                  type: scalar
            - name: street
              value:
                  type: scalar
-
    data: |
        ---
        city: New Orleans
        state: LA
        where ya got ya shoes: on ya feet, on Bourbon St.
    schema: *StreetCityState
    error: |
        Expected key 'street', got 'where ya got ya shoes'
-
    data: |
        ---
        banana: yellow
        carrot: orange
        people:
            - fname: al
              salary: 44
            - fname: bob
              salary: 33
    schema:
        type: map
        items:
            - name: banana
              value:
                type: scalar
            - name: carrot
              value:
                type: scalar
            - name: people
              value:
                type: seq
                child:
                    type: map
                    items:
                        - name: fname
                          value:
                            type: scalar
                        - name: salary
                          value:
                            type: scalar
"""

class ValidatingLoader:
    def load(self, data, schema):
        self.simulateParser(data)
        return self.loadData(schema)

    def loadData(self, schema):
        typ = self.parser.getType()
        return self._load(typ, schema)

    def _load(self, typ, schema):
        if typ != schema['type']:
            raise Exception("Wanted %s, got %s\n" % (schema['type'], typ))
        if typ == 'seq':
            return self._loadSeq(schema)
        if typ == 'map':
            return self._loadMap(schema)
        else:
            return self.parser.getScalar()

    def _loadSeq(self, schema):
        results = []
        cnt = 0
        max = schema.get('max', None)
        schema = schema['child']
        while 1:
            typ = self.parser.getType()
            if typ is None:
                return results
            else:
                cnt += 1
                self.checkMax(cnt, max)
                results.append(self._load(typ, schema))

    def _loadMap(self, schema):
        results = {}
        for item in schema['items']:
            self.parser.getType()
            name = self.parser.getScalar()
            self.checkName(name, item)
            value = self.loadData(item['value'])
            results[name] = value
        self.parser.getType()
        return results

    def checkMax(self, cnt, max):
            if max is not None and cnt > max:
                raise Exception("Seq has max %d elements\n" % max)

    def checkName(self, name, item):
        if name != item['name']:
            raise Exception("Expected key '%s', got '%s'\n" % \
                (item['name'], name))

    def simulateParser(self, data):
        # This is the huge hack to work around
        # not having a true pull parser
        self.parser = mockParser(oldYamlLoad(data))

def testRoundTrip(data, schema):
    expected = oldYamlLoad(data)
    obj = ValidatingLoader().load(data, schema)
    assertEquals(expected, obj)

def oldYamlLoad(data):
    return load(data).next()

def testOneCase(test):
    data = test['data']
    schema = test['schema']
    if test.has_key('error'):
        assertError(lambda: testRoundTrip(data, schema),
            test['error'])
    else:
        testRoundTrip(data, schema)

class Test(YamlTest.YamlTest):
    def testFromYaml(self):
        for test in load(testCases).next():
            testOneCase(test)

if __name__ == '__main__':
    import unittest
    unittest.main()



import yaml
from test import assertEquals
from here import flushLeft
from math import sqrt

"""
DUMPING OBJECTS: > 
    YAML has several ways of persisting objects.  One view of an 
    object is that it's nothing more than it's dictionary.  By 
    default, YAML emits self.__dict__ from objects.
"""

class SimpleObject:
    def __init__(self):
        self.x = 100
        self.y = 20
    
    def xyz(self):
        return self.x + self.y + self.z

def testDumpingDictionary():
    obj = SimpleObject()
    obj.z = 3
    output = yaml.dump(obj)
    expected = simpleObjectYaml()
    assertEquals(output, expected)

def simpleObjectYaml():
    return flushLeft("""
        --- !!__main__.SimpleObject
        x: 100
        y: 20
        z: 3
        """)
    

testDumpingDictionary()

"""
LOADING OBJECTS: > 
    Of course, we would expect to be able to load that
    YAML right back into Python, and we can.  You get 
    back an object of the intended class, with all of 
    the normal methods.  Be aware, though, that if your 
    class creates methods on the fly, or if it does 
    other trickery, then you might get unexpected results.
"""

def testLoadingDictionary():
    obj = yaml.load(simpleObjectYaml()).next()
    assertEquals(obj.xyz(), 123)

testLoadingDictionary()

"""
CUSTOM DUMPING: >
    Some times you want more control over how you dump
    objects in YAML.  You might not want to dump all 
    members of the object, for example.  Also, you may 
    not want to export the module name.
"""

class Triangle:
    def __init__(self,x,y):
        self.x = x
        self.y = y
        self._hypotneuse = sqrt(x*x + y*y)

    def to_yaml(self):
        # hide the hypotneuse attribute, it's private;
        # also use inches instead of feet
        view = {
            'x_inches': self.x * 12,
            'y_inches': self.y * 12
        }
        return (view, '!!triangle_in_inches')

def testDumpTriangle():
    triangle = Triangle(3,4)
    assertEquals(yaml.dump(triangle), triangleYaml())

def triangleYaml():
    return flushLeft("""
        --- !!triangle_in_inches
        x_inches: 36
        y_inches: 48
        """)

testDumpTriangle()

"""
CUSTOM LOADING: >
    You have control over the YAML load process too.  Normally
    YAML resolves private types for you automatically, but you
    can override its behavior.
"""

class MyResolver:
    def resolveType(self, data, typestring):
        if typestring == '!!triangle_in_inches':
            x = data['x_inches'] / 12
            y = data['y_inches'] / 12
            return Triangle(x,y)
        else:
            raise 'Private type %s not supported' % typestring

def testCustomLoad():
    obj = yaml.load(triangleYaml(),MyResolver()).next()
    assertEquals(obj._hypotneuse, 5.0)

testCustomLoad()

"""
USING FROM_YAML(): >
    Another way to customize the YAML loading process is to 
    supply a from_yaml() method.  For example, you might want
    the loading of an object to have some kind of a side effect.
    Or, you may need to calculate some value that is not part of 
    the attribute.
"""

class TestConfig:
    lastTester = None
    def from_yaml(self, args):
        self.tester = args['tester']
        self.hitcount = args['hitcount'] + 1
        TestConfig.lastTester = self.tester
        return self

def testFromYaml():
    yamlData = flushLeft("""
        --- !!__main__.TestConfig
        tester: showell
        hitcount: 42
        """)
    obj = yaml.load(yamlData).next()
    assertEquals(obj.hitcount, 43)
    assertEquals(TestConfig.lastTester, 'showell')

testFromYaml()



import yaml, string

####### RUNNING YAML AGAINST THE README AND CHANGELOG

readme = yaml.loadFile("README")
print "README"
for item in readme.next():
    print item
print "\n\nCONTRIBUTORS"
for person in readme.next()['contributors']:
    print "===%s===" % person['who']
    print person['why?']
    print
print "\n\n"
print "CHANGELOG:\n"
print list(yaml.loadFile("CHANGELOG"))
print "\n\n"

######## USING YAML INSIDE YOUR PROGRAM

testData = \
"""
program: PyYaml
author: Steve Howell
---
shopping list:
 - apple
 - banana
todo:
 - eat more fruit:
     - especially bananas!
     - good for you
 - write a better demo
"""

print "YAML INSIDE YOUR PROGRAM"
for x in yaml.load(testData):
   print repr(x)
print "\n\n"


######### YPATH STUFF

try:
    print "YPATH EXPERIMENTATION"
    data = yaml.load(testData)
    print yaml.ypath("/author",data.next()).next()
    print yaml.dump(yaml.ypath("/todo/0",data.next()).next())
except NotImplementedError:
    print "Experimental YPATH requires Python 2.2"

######### YAML DUMPER 

class Person:
    def __init__(self, fname, lname, salary, children):
        self.fname = fname
        self.lname = lname
        self.salary = salary
        self.children = children
        # private variables
        self._fullname = fname + ' ' + lname
        if salary:
            self._sal_per_month = salary / 12.0
        self._num_children = len(children)
    def to_yaml(self):
        return ({
            'first name': self.fname,
            'last name':  self.lname,
            'salary':     self.salary
        }, '!!Person')

mrBarson = Person('Foo', 'Barson', 20, ['ex', 'theomatic'])
mrDoe = Person('John', 'Doe', None, [])
print yaml.dump([mrBarson, mrDoe])

print "\n\nANOTHER WAY TO STDOUT:\n"
import sys
yaml.dumpToFile(sys.stdout, [mrBarson, mrDoe])

print "\n\nDUMP MULTIPLE DOCS TO A FILE:\n"
file = open('DEMO_OUTPUT.TXT', 'w')
yaml.dumpToFile(file, 
    {'source': "Demo output from demo.py"},
    [
        'apple',
        'banana',
    ],
    'Third document'      
)
file.close()





import time, re, string
from types import ListType, TupleType

PRIVATE_NOTICE = """
  This module is considered to be private implementation
  details and is subject to change.  Please only use the
  objects and methods exported to the top level yaml package.
"""

# 
# Time specific operations
#

_splitTime = re.compile('\-|\s|T|t|:|\.|Z')
matchTime = re.compile(\
          '\d+-\d+-\d+([\s|T|t]\d+:\d+:\d+.\d+(Z|(\s?[\-|\+]\d+:\d+)))?')

def _parseTime(val):
    if not matchTime.match(val): raise ValueError(val)
    tpl = _splitTime.split(val)
    if not(tpl): raise ValueError(val)
    siz = len(tpl)
    sec = 0
    if 3 == siz:
       tpl += [0,0,0,0,0,-1]
    elif 7 == siz:
       tpl.append(0)
       tpl.append(-1)
    elif 8 == siz:
       if len(tpl.pop()) > 0: raise ValueError(val)
       tpl.append(0)
       tpl.append(-1)
    elif 9 == siz or 10 == siz:
       mn = int(tpl.pop())
       hr = int(tpl.pop())
       sec = (hr*60+mn)*60
       if val.find("+") > -1: sec = -sec
       if 10 == siz: tpl.pop()
       tpl.append(0)
       tpl.append(-1)
    else:
       raise ValueError(val)
    idx = 0
    while idx < 9:
       tpl[idx] = int(tpl[idx])
       idx += 1
    if tpl[1] < 1 or tpl[1] > 12: raise ValueError(val)
    if tpl[2] < 1 or tpl[2] > 31: raise ValueError(val)
    if tpl[3] > 24: raise ValueError(val)
    if tpl[4] > 61: raise ValueError(val)
    if tpl[5] > 61: raise ValueError(val)
    if tpl[0] > 2038:
        #TODO: Truncation warning
        tpl = (2038,1,18,0,0,0,0,0,-1)
    tpl = tuple(tpl)
    ret = time.mktime(tpl)
    ret = time.localtime(ret+sec)
    ret = ret[:8] + (0,)
    return ret


class _timestamp:
    def __init__(self,val=None):
        if not val:
           self.__tval = time.gmtime()
        else:
           typ = type(val)
           if ListType == typ:
               self.__tval = tuple(val)
           elif TupleType == typ:
               self.__tval = val
           else:
               self.__tval = _parseTime(val)
           if 9 != len(self.__tval): raise ValueError
    def __getitem__(self,idx): return self.__tval[idx]
    def __len__(self): return 9
    def strftime(self,format): return time.strftime(format,self.__tval)
    def mktime(self):          return time.mktime(self.__tval)
    def asctime(self):  return time.asctime(self.__tval)
    def isotime(self):  
        return "%04d-%02d-%02dT%02d:%02d:%02d.00Z" % self.__tval[:6]
    def __repr__(self): return "yaml.timestamp('%s')" % self.isotime()    
    def __str__(self):  return self.isotime()
    def to_yaml_implicit(self): return self.isotime()
    def __hash__(self): return hash(self.__tval[:6]) 
    def __cmp__(self,other): 
        try:
            return cmp(self.__tval[:6],other.__tval[:6])
        except AttributeError:
            return -1

try: # inherit from mx.DateTime functionality if available
    from mx import DateTime
    class timestamp(_timestamp):
        def __init__(self,val=None):
            _timestamp.__init__(self,val)
            self.__mxdt = DateTime.mktime(self.__tval)
        def __getattr__(self, name):
              return getattr(self.__mxdt, name)
except:
    class timestamp(_timestamp): pass
        


def unquote(expr):
    """
        summary: >
           Simply returns the unquoted string, and the
           length of the quoted string token at the 
           beginning of the expression.
    """
    tok = expr[0]
    if "'" == tok: 
        idx = 1
        odd = 0
        ret = ""
        while idx < len(expr):
            chr = expr[idx]
            if "'" == chr:
                if odd: ret += chr
                odd = not odd
            else:
                if odd:
                    tok = expr[:idx]
                    break
                ret += chr
            idx += 1
        if "'" == tok: tok = expr
        return (ret,len(tok))
    if '"' == tok:
        idx = 1
        esc = 0
        while idx < len(expr):
            chr = expr[idx]
            if '"' == chr and not esc:
                tok = expr[:idx] + '"'
                break
            if '\\' == chr and not esc: esc = 1
            else: esc = 0
            idx += 1
        if '"' == tok:
            raise SyntaxError("unmatched quote: " + expr)
        ret = eval(tok)  #TODO: find better way to unquote
        return (ret,len(tok))
    return (expr,len(expr))



import YamlTest
from yaml.dump import *
import yaml
from here import flushLeft
from test import assertEquals

class SampleClass:
    def expected(self, modname, classname):
        """
        Tough to test, because this class may be in __main__ 
        namespace, may be in TestDumper, depending where you 
        run it
        """
        return """
        --- !!%s.%s
        bar: 4
        foo: 3
        items:
            - apple
            - banana
        """ % (modname, classname)

    def __init__(self):
        self.foo = 3
        self.bar = 4
        self.items = ['apple', 'banana']


class SampleException(Exception):
    def __init__(self, args):
        self.args = args

class Test(YamlTest.YamlTest):
    def dumpTest(self, obj, expect):
        assertEquals(yaml.dump(obj), flushLeft(expect))
    def dumpTestSort(self,sort):
        obj = {'a':'aaaa','d':'dddd','b':'bbbb','c':'cccc'}
        exp = flushLeft(\
            """
            ---
            b: bbbb
            d: dddd
            a: aaaa
            c: cccc
            """)
        dumper = yaml.Dumper().setSort(sort)
        self.assertEquals(dumper.dump(obj),exp)
    def testsort_map(self):
        self.dumpTestSort({'a':4,'b':2,'d':3})
    def testsort_tuple(self):
        self.dumpTestSort(('b','d','a'))
    def testsort_list(self):
        self.dumpTestSort(['b','d','a'])
    def testsort_func(self):
        self.dumpTestSort({'a':4,'b':2,'d':3}.get)
    def test1(self):
        self.dumpTest(
            { 'foo': 'bar', 'yo': 'mama' },
            """
            ---
            foo: bar
            yo: mama
            """,
        )

    def test2(self):
        self.dumpTest(
            { 'foo': {'bar': 3} },
            """
            ---
            foo:
                bar: 3
            """,
        )

    def test3(self):
        self.dumpTest(
            { 
                'foo': 3,
                'fruits': {
                    'apple': 'red',
                    'banana': 'yellow',
                }
            },
            """
            ---
            foo: 3
            fruits:
                apple: red
                banana: yellow
            """
            )
                
    def test4(self):
        self.dumpTest(
            { 
                'foo': 3,
                'fruits': ['apple', 'orange']
            },
            """
            ---
            foo: 3
            fruits:
                - apple
                - orange
            """
            )
                
    def testTuple(self):
        self.dumpTest(
            { 
                'foo': 3,
                'fruits': ('apple', 'orange')
            },
            """
            ---
            foo: 3
            fruits:
                - apple
                - orange
            """
            )
                
    def testMultiLineBlock(self):
        self.dumpTest(
             { 
                 'foo': "a\nb\nc\n",
             },
             """
             ---
             foo: |
                 a
                 b
                 c
             """
             )

    def testDoubleQuote(self):
        assertEquals(doubleUpQuotes("foo'bar"), "foo''bar")
        assertEquals(doubleUpQuotes("''x'"), "''''x''")

    def testQuotingRules(self):
        # Raw:
        self.assertEquals(dump(None), "--- ~\n")
        self.assertEquals(dump('simple'), "--- simple\n")
        self.assertEquals(dump(''), "--- ''\n")
        self.assertEquals(dump('two words'), "--- two words\n")
        self.assertEquals(dump('single\'quote'), "--- single'quote\n")
        self.assertEquals(dump('5.2'), "--- '5.2'\n")
        self.assertEquals(dump(5.2), "--- 5.2\n")
        self.assertEquals(dump(1234), "--- 1234\n")
        self.assertEquals(dump('harmless-dash'), "--- harmless-dash\n")
        self.assertEquals(dump('* harmless bullet'), "--- * harmless bullet\n")
        self.dumpTest(
            [ "a - b", "another-harmless-dash" ],
            """
            ---
            - a - b
            - another-harmless-dash
            """)
        # Single Quoted:
        self.assertEquals(dump(''), "--- ''\n")
        self.assertEquals(dump('4.3.1.5.2'), "--- '4.3.1.5.2'\n")
        self.assertEquals(dump("'leading quote"), '--- "\'leading quote"\n')
        self.assertEquals(dump('4.3.'), "--- '4.3.'\n")
        self.assertEquals(dump('12345'), "--- '12345'\n")
        self.assertEquals(dump('-12345'), "--- '-12345'\n")
        self.assertEquals(dump('key: value'), "--- 'key: value'\n")
        self.assertEquals(dump('ending colon:'), "--- 'ending colon:'\n")
        self.assertEquals(dump('&foo'), "--- '&foo'\n")
        self.assertEquals(dump(' xx'), "--- ' xx'\n")
        self.assertEquals(dump('*bar'), "--- '*bar'\n")
        assertEquals(dump('"middle \'quote'), '--- \'"middle \'\'quote\'\n')
        # Double quoted:
        self.assertEquals(dump("\thas a tab"), "--- \"\\thas a tab\"\n")
        self.dumpTest(
            { "key: quote": 'normal value' },
            """
            ---
            'key: quote': normal value
            """)

    def testArrayWithSingleQuoted(self):
        self.dumpTest(
            [ 'foo:', {'bar': 'colon:'}],
            """
            ---
            - 'foo:'
            -
                bar: 'colon:'
            """
            )

    def testDumpObject(self):
        sample = SampleClass()
        self.dumpTest(sample, 
            sample.expected(sample.__module__, 'SampleClass'))

    def testWithSpuriousToYaml(self):
        class ClassWithSpuriousToYaml(SampleClass):
            to_yaml = 1 # Shouldn't get invoked
        x = ClassWithSpuriousToYaml()
        self.dumpTest(x, 
            x.expected(x.__module__,'ClassWithSpuriousToYaml'))

    def testNormalToYamlUse(self):
        class NormalYamlUse(SampleClass):
            def to_yaml(self):
                dict = {}
                dict['bar'] = self.foo * 2
                return (dict, None)
        self.dumpTest(NormalYamlUse(),
            """
            ---
            bar: 6
            """)

    def testDumpingPrivateTypes(self):
        class Foobar(SampleClass):
            def to_yaml(self):
                dict = {'foo': 'bar'}
                return (dict, '!!foobar')
        self.dumpTest(Foobar(),
            """
            --- !!foobar
            foo: bar
            """)
        class OddList(SampleClass):
            def to_yaml(self):
                return ([1, 3, 5], '!!oddlist')
        self.dumpTest(OddList(),
            """
            --- !!oddlist
            - 1
            - 3
            - 5
            """)


    def testToYamlShouldBeAllowedToThrowIfItWants(self):
        class ThrowsInsideToYaml:
            def to_yaml(self):
                raise SampleException('bla')
        caught_exception = 'undef'
        try:
            dump(ThrowsInsideToYaml())
        except Exception, e:
            caught_exception = e
        self.assertEquals('bla', caught_exception.args)

    def testObjectWithListOfClasses(self):
        class Foo:
            def __init__(self, items):
                self.items = items
             
        class Bar:
            def __init__(self, data):
                self.data = data

        foo = Foo([Bar('apple'), Bar('banana')])
        mod = foo.__module__ # depends where called
        self.dumpTest(foo,
            """
            --- !!%s.Foo
            items:
                - !!%s.Bar
                    data: apple
                - !!%s.Bar
                    data: banana
            """ % (mod, mod, mod))

    def testHasMethod(self):
        class ClassWithMethodA:
            b = 1
            def a(): pass

        sc = ClassWithMethodA()
        self.assertEquals(1, hasMethod(sc, 'a'))
        self.assertEquals(0, hasMethod(sc, 'b'))

    def testComplexKey(self):
        self.dumpTest( {(3,4): 4},
            """
            ---
            ?
                - 3
                - 4
            : 4
            """)            
    
    def testApostrophe(self):
        self.dumpTest( "Joe's hot dogs.\n",
            """
            --- |
            Joe's hot dogs.
            """)

    def testCustomIndent(self):
        self.assertEquals(
            yaml.Dumper().setIndent('xx').dump(
                { 
                    'foo': 3,
                    'fruits': ('apple', 'orange')
                }, 
            ),
            flushLeft(
                """
                ---
                foo: 3
                fruits:
                xx- apple
                xx- orange
                """
            )
            )

    def testUnicode(self):
        if YamlTest.hasUnicode:
            self.dumpTest({'foo': u"Foo\u263A"},
                r"""
                ---
                foo: "Foo\u263a"
                """)

    def testTab(self):
        self.dumpTest({'tab': "foo\tbar"},
            r"""
            ---
            tab: "foo\tbar"
            """)

    def testIsMulti(self):
        self.failUnless(isMulti("foo\nbar"))
        self.failIf(isMulti("foobar"))
        self.failIf(isMulti("line\twith tab\nsecond line"))
        self.failUnless(isMulti("foo\\slash\nbar"))

    def testHasSpecialChar(self):
        self.failUnless(hasSpecialChar("foo\tbar"))
        self.failIf(hasSpecialChar("foobar"))

    def testTabs(self):
        self.dumpTest({'control': "\b1998\t1999\t2000\n"},
            r"""
            ---
            control: "\b1998\t1999\t2000\n"
            """)

    def test12345(self):
        self.dumpTest({'foo': 12345, 'bar': '12345'},
            r"""
            ---
            bar: '12345'
            foo: 12345
            """)

    def testDataThatsCoincidentallyTheSame(self):
        self.dumpTest([ {'foo': 'bar'}, {'foo': 'bar'} ],
            """
            ---
            -
                foo: bar
            -
                foo: bar
            """)

    def testAliasClass(self):
        dup = {'foo': 'bar'}
        dupList = [dup, dup]
        myAnchors = YamlAnchors(dupList)
        self.assertEquals(id(dup), myAnchors._anchorVisits.keys()[0])
        self.assertEquals(0,myAnchors.isAlias(dup))
        self.assertEquals(1,myAnchors.shouldAnchor(dup) )
        self.assertEquals(0,myAnchors.shouldAnchor('bar'))
        self.assertEquals(0,myAnchors.shouldAnchor(dup['foo']))
        self.assertEquals(0,myAnchors.shouldAnchor(dup))
        self.assertEquals(1,myAnchors.isAlias(dup))

    def testAliases(self):
        dup = {'foo': 'bar'}
        self.dumpTest([dup, dup],
            """
            ---
            - &1
                foo: bar
            - *1
            """)

    def testHashAlias(self):
        dup = {}
        dup['foo'] = dup
        self.dumpTest(dup, 
            """
            --- &1
            foo: *1
            """)

    def testEmptyArray(self):
        self.dumpTest([],
            """
            --- []
            """)

    def testEmptyHash(self):
        self.dumpTest({},
            """
            --- {}
            """)

    def testEmptyHashAsHashKey(self):
        self.dumpTest({'foo': {} },
            """
            ---
            foo: {}
            """)

    def testEmptyArrrayAsHashKey(self):
        self.dumpTest({'foo': [] },
            """
            ---
            foo: []
            """)

    def testAliasNone(self):
        foo = None
        self.dumpTest([foo, foo, foo],
            """
            ---
            - ~
            - ~
            - ~
            """)

    def testDumpMany(self):
        assertEquals(
            yaml.dump({'foo': 1}, ['a', 'b', 'c']),
            flushLeft(
                """
                ---
                foo: 1
                ---
                - a
                - b
                - c
                """))

    def testDumpExtraNewLines(self):
        self.dumpTest( "Joe's hot dogs.\n\n",
            """
            --- |+
            Joe's hot dogs.

            """)


    def testDumpExtraNewLinesInDict(self):
        data = {'preserving': "extra new lines are kept\n\n\n"}
        self.dumpTest(data,
            """
            ---
            preserving: |+
                extra new lines are kept
                
                
            """)

if __name__ == '__main__':
    import unittest
    unittest.main()



from PyYaml import yaml
from test import assertEquals
from ypath import Ypath
import re

def Insert(data, command):
    if command.has_key('ypath'):
        Ypath(data, command['ypath']).append(command['value'])
        return data
    return data + [command['value']]

def TestInsert(data, command, expected):
    assertEquals(Insert(data, command), expected)    

doc = yaml.loadFile("insert.yml")[0]
for test in doc['tests']:
    if not test.has_key('ignore'):
        TestInsert(test['data'], test['command'], test['expected'])





def assertEquals(first, second, msg=''):
    if first != second:
        raise AssertionError, \
              (msg + "\n%s !=\n%s" % (`first`, `second`))

def assertError(func, expectedError):
    raised = 1
    try:
        func()
        raised = 0
    except Exception, e:
        assertEquals(str(e), expectedError)
    except:
        raise "Unexpected exception"
    if raised == 0:
        raise "Never threw exception"





import profile
import pstats
import yaml
import glob

def loadAllFiles():
    testFiles = glob.glob('TestingSuite/*.yml')
    for file in testFiles:
        docs = yaml.loadFile(file)
        for doc in docs:
            exercise(doc)

def exercise(doc):                
    if 'python' in doc:
        if 'yaml' in doc:
            for subDoc in yaml.load(doc['yaml']):
                pass
print dir(profile)
profile.run('loadAllFiles()', 'profileResults')

p = pstats.Stats('profileResults')
file = open("profile.out", "w")
p.sort_stats('time').print_stats(25)
